package com.jproxy.samples;

import javax.naming.*;
import java.util.*;
import com.jproxy.samples.interfaces.*;
import com.jproxy.proxy.*;


public class SampleInitializer
	implements Runnable
{
	public static SampleInitializer initializer = null;

	public boolean isRMIInitialized = false;
	public boolean isCORBAInitialized = false;
	public String corbaMessage = "";
	public String rmiMessage = "";

	org.omg.CORBA.ORB orb = null;
	Thread thread = null;

	public SampleInitializer()
	{
		initRMITest();
		initCORBATest();
	}

	static public void main(String[] args)
	{
		if(initializer!=null)
			return;
		initializer = new SampleInitializer();
	}

	void initRMITest()
	{
		if(isRMIInitialized)
			return;
		try {
			// Create object to be bound
			ITest test = new com.jproxy.samples.rmi.test.TestImpl();

			// Create the initial context
			InitialContext context = new InitialContext();

			context.rebind("test", test);

			test = (ITest) context.lookup("test");

			System.out.println(
				"\r\n**********************************"+
				"\r\nRMI Test "+
				"\r\nServer Time: "+(new Date(test.getServerTime()))+
				"\r\nServer started. "+
				"\r\n**********************************\r\n");

			// Close the context
			context.close();
			isRMIInitialized = true;
		}
		catch (Exception e){
			e.printStackTrace();
			rmiMessage = Helper.getStackTrace(e);
		}
	}


	void initCORBATest()
	{
		if(isCORBAInitialized)
			return;
		try{
/* Sun CORBA
*			props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");
*			props.put("org.omg.CORBA.ORBSingletonClass", "com.sun.CORBA.idl.ORBSingleton");
*/
/* JacORB CORBA
*			props.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
*			props.put("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
*			props.put("custom.props", "/opt/jboss/server/default/conf/jacorb.properties");
*/

			String[] args = new String[]{"-ORBInitialPort", "1050"};

			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
			System.out.println("ORB: "+orb.getClass().getName());

			org.omg.PortableServer.POA rootpoa = org.omg.PortableServer.POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();

			// Create the servant and register it with the ORB
			com.jproxy.samples.corba.test.TestImpl impl = new com.jproxy.samples.corba.test.TestImpl();
			impl.setORB(orb);

			// get object reference from the servant
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(impl);
			com.jproxy.samples.corba.test.Test href = com.jproxy.samples.corba.test.TestHelper.narrow(ref);

			// get the root naming context
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
			// Use NamingContextExt which is part of the Interoperable
			// Naming Service (INS) specification.
			org.omg.CosNaming.NamingContextExt ncRef = org.omg.CosNaming.NamingContextExtHelper.narrow(objRef);

			// bind the Object Reference in Naming
			org.omg.CosNaming.NameComponent path[] = ncRef.to_name("test");
			ncRef.rebind(path, href);

			System.out.println(
				"\r\n**********************************"+
				"\r\nCORBA Test "+
				"\r\nServer Time: "+(new Date(impl.getServerTime()))+
				"\r\nServer started. "+
				"\r\n**********************************\r\n");

			thread = new Thread(this, "SampleInitializer");
			thread.start();
			isCORBAInitialized = true;
		}
		catch (Exception e){
			e.printStackTrace();
			corbaMessage += Helper.getStackTrace(e);
		}
	}


	public void run()
	{
		try
		{
			// wait for invocations from clients
			orb.run();
		}
		catch (Exception e){
			e.printStackTrace();
			corbaMessage += Helper.getStackTrace(e);
		}
	}
}
