package com.jproxy.samples.ejb.test;

import javax.jms.*;
import javax.naming.*;
import java.util.*;
import java.rmi.RemoteException;

import com.jproxy.samples.interfaces.ITest;
import com.jproxy.samples.interfaces.ITestCallback;

//	command line:
// java -cp samples.jar;proxyclient.jar;ejb.jar com.jproxy.samples.ejb.test.SessionClient [http | https://]YourServer[:Port]

public class JMSPerformanceClient
	extends PerformanceClient
	implements ITest, MessageListener
{
	ITestCallback callback = null;

	Context context = null;
	Properties env = new Properties();
	Test test = null;
	byte[] bytes = null;

	TopicConnectionFactory connectionFactory = null;
	TopicConnection connection = null;
	Topic topic = null;
	TopicSession topicSession = null;
	TopicPublisher publisher = null;
	TopicSubscriber subscriber = null;
	MapMessage message = null;

	boolean gotResponse = false;
	String clientId = ""+(int)(Math.random() * Integer.MAX_VALUE);


	public void onMessage(Message message)
	{
		try
		{
			MapMessage msg = (MapMessage)message;
			bytes = msg.getBytes("bytes");
		}
		catch(JMSException e){
			e.printStackTrace();
		}
		finally{
			gotResponse = true;
			synchronized(this){
				this.notifyAll();
			}
			try{
				callback.onCallback(bytes);
			}
			catch(RemoteException e){
				e.printStackTrace();
			}
		}
	}

	public long getServerTime()
		throws RemoteException
	{
		return test.getServerTime();
	}

	public byte[] echoBytes(byte[] bytes)
		throws RemoteException
	{
		try{
			connection.stop();
			message.setStringProperty("isCallback", "false");
			message.setBytes("bytes", bytes);
			publisher.publish(message);
		}
		catch(Exception e){
			e.printStackTrace();
			throw new RemoteException(e.getMessage());
		}
		return bytes;
	}

	public void callbackBytes(Object callback, int numberOfBytes, int numberOfLoops)
		throws RemoteException
	{
		bytes = null;
		this.callback = (ITestCallback)callback;

		Hashtable h = new Hashtable();
		try{
			connection.start();
			message.setStringProperty("isCallback", "true");
			message.setInt("numberOfBytes", numberOfBytes);
			message.setInt("numberOfLoops", numberOfLoops);
			publisher.publish(message);
			synchronized(this){
				this.wait(10000);
			}
			if(gotResponse)
				System.out.println("[JMSPerformanceClient] callbackBytes: got asynchronous response");
		}
		catch(Exception e){
			e.printStackTrace();
			throw new RemoteException(e.getMessage());
		}
	}

	public void init()
		throws Exception
	{
		env.put(Context.PROVIDER_URL, hostUrl);
		env.put(Context.INITIAL_CONTEXT_FACTORY, "com.jproxy.proxy.NamingContextFactory");

		/** The line below is commented because the Linux KDE Konqueror Web Browser does not support
		* a standard way of obtaining InitialContext because of wrong definition of Java sand-box security.
		* For the browser "getClassLoader" permission is not allowed for sand-box.
		*/
		// context = new InitialContext(env);
		javax.naming.spi.InitialContextFactory factory = new com.jproxy.proxy.NamingContextFactory();
		context = factory.getInitialContext(env);

		System.out.println("Context.lookup(\"ConnectionFactory\");");
		TopicConnectionFactory connectionFactory =
		(TopicConnectionFactory)context.lookup("ConnectionFactory");
		System.out.println("TopicConnectionFactory.createTopicConnection();");
		connection = connectionFactory.createTopicConnection();

		System.out.println("Context.lookup(\"topic/TestMessageDrivenBean\");");
		topic = (Topic)context.lookup("topic/TestMessageDrivenBean");

		context.close();

		System.out.println("TopicConnection.createTopicSession(false, javax.jms.Session.DUPS_OK_ACKNOWLEDGE);");
		topicSession = connection.createTopicSession(false, javax.jms.Session.DUPS_OK_ACKNOWLEDGE);

		System.out.println("TopicSession.createPublisher("+topic+");");
		publisher = topicSession.createPublisher(topic);

		String selector = "clientId LIKE '"+clientId+"' AND isCallback LIKE 'true'";
		subscriber = topicSession.createSubscriber(topic, selector, false);
		System.out.println("TopicSession.createSubscriber("+topic+");");
		subscriber.setMessageListener(this);

		message = topicSession.createMapMessage();
		message.setStringProperty("clientId", clientId);

		session = this;

		isInitialized = true;
	}


	public void destroy()
	{
		if(!isInitialized)
			return;
		if(connection==null)
			return;
		try{
			connection.close();
		}
		catch(JMSException e){}
		connection = null;
	}
}
