package com.jproxy.samples.ejb.test;
/*
usage:
  java -cp samples.jar;proxyclient.jar [-Djava.properties] \
    com.jproxy.samples.ejb.test.PerformanceClient \
	[mode=EJB | RMI | CORBA | SOAP ] [loops=1] [bytes=0] [threads=1] [url=localhost]
where:
    loops - number of loops. Default 1
    bytes - number of bytes send(received) to host in request. Default 0
    threads - number of threads. Default 1
    url - Will try \"localhost\"
where:
 #_of_loops - number of loops. Default 1"+
 #_of_bytes - number of bytes send(received) to host in request. Default 0"+
 #_of_threads - number of threads. Default 1"+

usage example: Performance test WITH JProxy
10 loops, 10000 bytes, 100 threads (10*100 calls, 10*100*10000 sent bytes, 10*100*10000 received)
%JAVA_HOME%\bin\java -cp samples.jar;proxyclient.jar;j2api.jar \
 -Djava.naming.factory.initial=com.jproxy.proxy.NamingContextFactory \
 -Djava.naming.provider.url=localhost:8080 \
 com.jproxy.samples.ejb.test.PerformanceClient \
 loops=10 bytes=10000 threads=100

usage example: Performance test WITHOUT JProxy (JBoss)
10 loops, 10000 bytes, 100 threads (10*100 calls, 10*100*10000 sent bytes, 10*100*10000 received)
%JAVA_HOME%\bin\java -cp samples.jar; \
 %LIB_PATH%/jboss-j2ee.jar;%LIB_PATH%/jboss-client.jar;\
 %LIB_PATH%/jnp-client.jar;%LIB_PATH%/jbosssx-client.jar;\
 %LIB_PATH%/jboss-common-client.jar;%LIB_PATH%/log4j.jar \
 -Djava.naming.factory.initial=org.jnp.interfaces.NamingContextFactory \
 -Djava.naming.provider.url=localhost \
 com.jproxy.samples.ejb.test.PerformanceClient \
 10 10000 100
*/
import javax.naming.*;
import java.util.*;
import java.io.*;

import com.jproxy.proxy.*;
import com.jproxy.samples.interfaces.ITest;
import com.jproxy.samples.interfaces.ITestCallback;

public class PerformanceClient
	implements Runnable, ITestCallback
{
	static final String JPROXY_PERFORMANCE = "JPROXY";
	static final String EJB_PERFORMANCE = "EJB";
	static final String RMI_PERFORMANCE = "RMI";
	static final String CORBA_PERFORMANCE = "CORBA";
	static final String SOAP_PERFORMANCE = "SOAP";
	static final String JMS_PERFORMANCE = "JMS";

	boolean isTestCompleted = false;
	static private Boolean sync = new Boolean(true);

	public ITest session = null;
	public boolean isInitialized = false;

	public String hostUrl = "localhost";
	String mode = JPROXY_PERFORMANCE;
	boolean isCallback = false;
	int bytes = 0;
	int loops = 1;

	int loop = 0;
	int totalLoops = 0;
	int index = 0;
	float delay = 0;
	float minDelay = 0;
	float maxDelay = 0;
	long startTime = 0;
	int threads = 0;
	byte[] buf = null;
	Thread thread = new Thread(this);
	String error = null;
	static public PrintStream out = System.out;
	static Hashtable env = new Hashtable();

	public PerformanceClient()
	{
	}


	public void init()
		throws Exception
	{
	}

	public void destroy()
	{
		isInitialized = false;
	}

	public /*void*/Object onCallback(Object obj)
		throws java.rmi.RemoteException
	{
		totalLoops++;
		loop++;
		long tt = System.currentTimeMillis();
		float d = (float)((System.currentTimeMillis()-tt)/1000.0);
		if(d<minDelay)
			minDelay = d;
		else if(d>maxDelay)
			maxDelay = d;

		if(loop>=loops)
		{
			delay = (float)((System.currentTimeMillis()-startTime)/1000.0);
			isTestCompleted = true;
		}
		return obj;
	}

	void fillBuffer()
	{
		/* to test remote exceptions thrown by EJB:  if negative, set to "null" */
		if(bytes<0)
		{
			buf = null;
			return;
		}
		buf = new byte[bytes];
		for(int i=0; i<bytes; i++)
			buf[i] = (byte)(i & 0xFF);
	}

	public void testCallback()
		throws Exception
	{
		isTestCompleted = false;
		startTime = System.currentTimeMillis();
		session.callbackBytes(this, bytes, loops);
	}

	public void test()
		throws Exception
	{
		isTestCompleted = false;
		fillBuffer();
		startTime = System.currentTimeMillis();

		for(int i=0; i<loops; i++)
		{
			long tt = System.currentTimeMillis();
			session.echoBytes(buf);
			float d = (float)((System.currentTimeMillis()-tt)/1000.0);
			if(d<minDelay)
				minDelay = d;
			else if(d>maxDelay)
				maxDelay = d;
			loop++;
		}
		delay = (float)((System.currentTimeMillis()-startTime)/1000.0);
		out.println(statistics());
		isTestCompleted = true;
	}

	String statistics()
	{
		return
			"\r\n*******************************************************"+
			"\r\nThread number______________________________"+index+
			statistics(mode, isCallback, hostUrl, loops, bytes, threads, minDelay, maxDelay, delay/(loops*threads), delay)+
			"\r\n*******************************************************";
	}

	static String statistics(	String mode,
								boolean isCallback,
								String hostUrl,
								int loops,
								int bytes,
								int threads,
								float minDelay,
								float maxDelay,
								float averageDelay,
								float delay)
	{
		return
			"\r\nTest Mode__________________________________"+mode+
			"\r\nCommunication type_________________________"+(isCallback?"Asynchronous":"Synchronous")+
			"\r\nURL________________________________________"+hostUrl+
			"\r\nNumber of loops____________________________"+loops+
			"\r\nNumber of sent and received bytes per req._"+bytes+
			"\r\nTotal Number of sent and received bytes____"+bytes*threads*loops+
			"\r\nMin request time___________________________"+minDelay+" sec."+
			"\r\nMax request________________________________"+maxDelay+" sec."+
			"\r\nAverage request time_______________________"+averageDelay+" seconds"+
			"\r\nTotal time_________________________________"+delay+" seconds";
	}






	static void println(Object msg)
	{
		synchronized(sync)
		{
			System.out.println(msg);
		}
	}

	public void run()
	{
		try{
			init();
			if(isCallback)
				testCallback();
			else
				test();
			destroy();
		}
		catch(Throwable e){
			isTestCompleted = true;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			e.printStackTrace();
			e.printStackTrace(new PrintWriter(baos));
			error = new String(baos.toByteArray());
			out.println("\r\n########### ERROR in thread "+index+"###########\r\n"+error);
		}
	}


	static public Properties parseArgs(String[] args)
	{
		Properties res = new Properties();

		for(int i=0; i<args.length; i++)
		{
			String s = args[i];
			if(s.startsWith("-") && s.length()>1 && !s.startsWith("-D"))
				res.setProperty(s.substring(0, 2), s.substring(2));
			else
			{
				int n = s.indexOf("=");
				if(n!=-1)
					res.setProperty(s.substring(0, n), s.substring(n+1));
			}
		}
		return res;
	}


	public void validateArgs(Properties props)
		throws Exception
	{
		String errMsg = "";

		for(Enumeration en = props.keys(); en.hasMoreElements();)
		{
			String key = (String)en.nextElement();
			String val = props.getProperty(key);
			if(key.startsWith("-D"))
			{
				key = key.substring(2);
				env.put(key, val);
//				System.setProperty(key, val);
				System.out.println("key = "+key+" val = "+val);
				continue;
			}

			if(key.equalsIgnoreCase("loops"))
			{
				try{
					loops = Integer.parseInt(val);
					if(loops<=0 || loops>1000)
						throw new Exception();
				}catch(Exception e){
					errMsg += "Wrong \"loops\" parameter! "+val+"\r\n";
				}

			}

			if(key.equalsIgnoreCase("bytes"))
			{
				try{
					bytes = Integer.parseInt(val);
					if(bytes<0 || bytes>1000000)
						throw new Exception();
				}catch(Exception e){
					errMsg += "Wrong \"bytes\" parameter! "+val+"\r\n";
				}
			}

			if(key.equalsIgnoreCase("threads"))
			{
				try{
					threads = Integer.parseInt(val);
					if(threads<=0 || threads>10)
						throw new Exception();
				}catch(Exception e){
					errMsg += "Wrong \"threads\" parameter! "+val+"\r\n";
				}
			}

			if(key.equalsIgnoreCase("url"))
				hostUrl = val;

			if(key.equalsIgnoreCase("mode"))
				mode = val.toUpperCase();

			if(key.equalsIgnoreCase("callback"))
				isCallback = new Boolean(val.toUpperCase()).booleanValue();
		}
		if(errMsg.length()>0)
			throw new Exception(errMsg);

	}

	static public void executeTest(boolean isCallback, String mode, String hostUrl, int loops, int bytes, int threads)
	{
		String className = null;
		if(mode.equals("EJB"))
			className = "com.jproxy.samples.ejb.test.EJBPerformanceClient";
		else if(mode.equals("SOAP"))
			className = "com.jproxy.samples.soap.test.SOAPPerformanceClient";
		else if(mode.equals("CORBA"))
			className = "com.jproxy.samples.corba.test.CORBAPerformanceClient";
		else if(mode.equals("RMI"))
			className = "com.jproxy.samples.rmi.test.RMIPerformanceClient";
		else if(mode.equals("JMS"))
			className = "com.jproxy.samples.ejb.test.JMSPerformanceClient";
		else if(mode.equals("JPROXY"))
			className = "com.jproxy.samples.ejb.test.JProxyPerformanceClient";
		else return;

		try
		{
			Class clientClass = Class.forName(className);
			PerformanceClient[] clients = new PerformanceClient[threads];
			long time = System.currentTimeMillis();
			for(int t=0; t<threads; t++)
			{
				clients[t] = (PerformanceClient)clientClass.newInstance();
				clients[t].isCallback = isCallback;
				clients[t].mode = mode;
				clients[t].hostUrl = hostUrl;
				clients[t].loops = loops;
				clients[t].bytes = bytes;
				clients[t].isTestCompleted = false;
				clients[t].index = t;
//				clients[t].run();
				clients[t].thread.start();
			}
			for(boolean isCompleted=false; !isCompleted;)
			{
				isCompleted = true;
				for(int t=0; t<threads; t++)
					if(!clients[t].isTestCompleted)
						isCompleted = false;
				Thread.currentThread().sleep(1);
			}

			/* below code can work only for synchronous calls
			for(int t=0; t<threads; t++)
				if(clients[t].thread.isAlive())
					clients[t].thread.join();
			*/
			float totalTime = (float)((System.currentTimeMillis()-time)/1000.0);

			int errorIndex = -1;
			for(int t=0; t<threads; t++)
			{
				if(clients[t]!=null && clients[t].error!=null)
				{
					errorIndex = t;
					break;
				}
			}
			if(errorIndex==-1)
			{
				float minDelay = 0;
				float maxDelay = 0;
				float averageDelay = 0;

				for(int t=0; t<threads; t++)
				{
					if(minDelay>clients[t].minDelay)
						minDelay = clients[t].minDelay;
					else if(maxDelay<clients[t].maxDelay)
						maxDelay = clients[t].maxDelay;
					averageDelay += clients[t].delay/loops;
				}
				averageDelay /= threads;
				String msg =
					"\r\n*****************************************************"+
					"\r\nTest executed seccessfully. No errors."+
					"\r\nNumber of threads____________________________"+threads+
					statistics(mode, isCallback, hostUrl, loops, bytes, threads, minDelay, maxDelay, averageDelay, totalTime)+
					"\r\n*****************************************************";
				out.println(msg);
			}
			else
				out.println("Error during test execution!\r\n"+clients[errorIndex].error);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}


	public static void main(String[] args)
	{
		Properties props = parseArgs(args);

		// the object is used only for keeping test parameters and passing them to executeTest(...)
		PerformanceClient client  = new PerformanceClient();
		try{
			String url = props.getProperty("-Djava.naming.provider.url");
			if(url!=null)
				client.hostUrl = url;

			client.validateArgs(props);
		}
		catch(Exception e){
			System.err.println(e.getMessage() +
				"usage: \r\njava -cp samples.jar;proxyclient.jar;j2api.jar [-Djava.properties] "+
				"com.jproxy.samples.ejb.test.PerformanceClient "+
				"[mode=EJB | RMI | CORBA | SOAP ] [loops=1] [bytes=0] [threads=1] [url=localhost]"+
				"\r\nloops - number of loops. Default 1"+
				"\r\nbytes - number of bytes send(received) to host in request. Default 0"+
				"\r\nthreads - number of threads. Default 1"+
				"\r\nWill try \"localhost\" ...");
			System.exit(1);
		}
		executeTest(client.isCallback, client.mode, client.hostUrl, client.loops, client.bytes, client.threads);
	}
}