package com.jproxy.samples.ejb.test;

/**
 * The class is a client for TestSession EJB example.
 * The request current time from server and prints it.
 * The class may be used as an Applet or stand-alone application.
 */

/*
usage:
  java -cp samples.jar;proxyclient.jar;ejb.jar com.jproxy.samples.ejb.test.SessionClient [http | https://]YourServer[:Port]

Here are some usage examples.
You have to correct paths for your environment.

For HTTPS test
  -Djava.protocol.handler.pkgs=com.sun.net.ssl.internal.www.protocol \
  -Djavax.net.ssl.trustStore=your_keystore \
  -Djavax.net.ssl.trustStorePassword=123456 \
  -Djavax.net.debug=all \

For EJB security test add to command line
  -Djava.naming.security.principal=admin \
  -Djava.naming.security.credentials=123 \

1. Any EJB Server with JProxy and EJB security:
%JAVA_HOME%\bin\java -cp samples.jar;proxyclient.jar;j2api.jar \
  -Djava.naming.factory.initial=com.jproxy.proxy.NamingContextFactory \
  -Djava.naming.provider.url=localhost:8080 \
  com.jproxy.samples.ejb.test.SessionClient

...And below you can see how many libraries needed to run the Client without JProxy!

2. JBoss 3.0 without JProxy:
%JAVA_HOME%\bin\java
  -cp samples.jar;%LIB_PATH%/jboss-j2ee.jar;%LIB_PATH%/jboss-client.jar;\
  %LIB_PATH%/jnp-client.jar;%LIB_PATH%/jbosssx-client.jar;\
  %LIB_PATH%/jboss-common-client.jar;%LIB_PATH%/log4j.jar \
  -Djava.naming.factory.initial=org.jnp.interfaces.NamingContextFactory \
  -Djava.naming.provider.url=localhost \
  com.jproxy.samples.ejb.test.SessionClient

3. Orion without JProxy:
%JAVA_HOME%\bin\java
  -cp samples.jar;proxyclient.jar;j2api.jar;%LIB_PATH%/orion.jar;%LIB_PATH%/jaas.jar;
  %LIB_PATH%/crimson.jar;%LIB_PATH%/jaxp.jar;%LIB_PATH%/jta.jar;%LIB_PATH%/jdbc.jar \
  -Djava.naming.factory.initial=com.evermind.server.ApplicationClientInitialContextFactory \
  -Djava.naming.provider.url=ormi://localhost/proxy \
  com.jproxy.samples.ejb.test.SessionClient
*/

import javax.naming.*;
import java.util.*;

import com.jproxy.proxy.*;
import com.jproxy.samples.interfaces.ITest;

public class SessionClient
{
	public String hostUrl = "localhost";
	public ITest session = null;
	public InitialContext context = null;
	public Properties env = new Properties();
	public boolean isInitialized = false;

	long time = 0;

	//Construct the applet
	public SessionClient()
	{
	}

	public void init()
		throws Exception
	{
		hostUrl = env.getProperty(Context.PROVIDER_URL);

		if(hostUrl!=null && hostUrl.toLowerCase().startsWith("https://"))
		{
			java.security.Provider provider = (java.security.Provider)Class.forName("com.sun.net.ssl.internal.ssl.Provider").newInstance();
			java.security.Security.addProvider(provider);
			/**  Uncomment if you want to implement your HostnameVerifier for HTTPS */
			/* com.sun.net.ssl.HttpsURLConnection.setDefaultHostnameVerifier(
			*      new com.sun.net.ssl.HostnameVerifier()
			*      {
			*          public boolean verify(String urlHostname, String certHostname)
			*          {
			*              return urlHostname.equals(certHostname);
			*          }
			*      });
			*/
		}

			/* JBoss without JProxy *
			/* env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
			*  env.put(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
			*/

			/* For EJB security test */
			/* env.put(Context.SECURITY_PRINCIPAL, "admin");
			*  env.put(Context.SECURITY_CREDENTIALS, "123");
			*/
			context = new InitialContext(env);

			ISessionHome sessionHome = (ISessionHome)context.lookup("TestSessionBean");
			/*	or
			* ISessionHome sessionHome = (ISessionHome)context.lookup("java:comp/env/TestSessionBean");
			*/
			context.close();
			session = sessionHome.create();
			isInitialized = true;
	}


	public void test()
		throws Exception
	{
			time = session.getServerTime();
			System.out.println("\r\n<<<<<< Server Time: "+(new Date(time))+" >>>>>\r\n");
	}

	public void destroy()
	{
		isInitialized = false;
		try{
			if(session instanceof ISession)
				((ISession)session).remove();
		}catch(Exception e){
//			e.printStackTrace();
		}
	}


	public static void main(String[] args)
	{
		SessionClient client = new SessionClient();
		client.env = System.getProperties();

		if(client.env.getProperty(Context.PROVIDER_URL)==null)
			client.env.setProperty(Context.PROVIDER_URL, "localhost");

		if(client.env.getProperty(Context.INITIAL_CONTEXT_FACTORY)==null)
			client.env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "com.jproxy.proxy.NamingContextFactory");

		try{
			client.init();
			client.test();
			client.destroy();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}