package com.jproxy.samples.ejb.test;

import javax.naming.*;
import java.util.*;
import java.rmi.RemoteException;
import javax.ejb.*;
import javax.jms.*;

public class TestMessageDrivenBean
	implements MessageDrivenBean, MessageListener
{
	private MessageDrivenContext mContext;

	private Context context = null;
	private TopicConnectionFactory connectionFactory = null;
	private TopicConnection connection = null;
	private Topic topic = null;
	private TopicSession topicSession = null;
	private TopicPublisher publisher = null;

	private String clientId = null;

	public void onMessage(Message message)
	{
		System.out.println("############# TestMessageDrivenBean.onMessage()");

		try
		{
			MapMessage msg = (MapMessage) message;
			clientId = msg.getStringProperty("clientId");
			if(msg.getStringProperty("isCallback").equals("false"))
			{
				byte[] bytes = msg.getBytes("bytes");
				System.out.println("[JProxy JMS Test] onMessage: echoBytes: bytes: "+bytes.length);
			}
			else
			{
				int numberOfBytes = msg.getInt("numberOfBytes");
				int numberOfLoops = msg.getInt("numberOfLoops");
				System.out.println("[JProxy JMS Test] onMessage: callbackBytes: numberOfBytes: "+numberOfBytes+" numberOfLoops: "+numberOfLoops);
				sendMessage(numberOfBytes, numberOfLoops);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}


	void sendMessage(int numberOfBytes, int numberOfLoops)
		throws Exception
	{
		System.out.println("[JProxy JMS Test] callbackBytes: bytes: "+numberOfBytes+" loops: "+numberOfLoops);
		byte[] buf = new byte[numberOfBytes];
		for(int i=0; i<numberOfBytes; i++)
			buf[i] = (byte)(i & 0xFF);

		for(int i=0; i<numberOfLoops; i++)
		{
			System.out.println("[JProxy JMS Test] callbackBytes: sending ObjectMessage. loop: "+i);
			sendMessage(buf);
		}
	}

	void sendMessage(byte[] bytes)
		throws Exception
	{
		if(connection==null)
			init();

		MapMessage message = topicSession.createMapMessage();
		message.setStringProperty("clientId", ""+clientId);
		message.setStringProperty("isCallback", "false");
		message.setBytes("bytes", bytes);
		publisher.publish(message);
	}


	public void init()
		throws Exception
	{
		context = new InitialContext();

		System.out.println("Context.lookup(\"ConnectionFactory\");");
		TopicConnectionFactory connectionFactory =
		(TopicConnectionFactory)context.lookup("ConnectionFactory");
		System.out.println("TopicConnectionFactory.createTopicConnection();");
		connection = connectionFactory.createTopicConnection();

		System.out.println("Context.lookup(\"topic/TestMessageDrivenBean\");");
		topic = (Topic)context.lookup("topic/TestMessageDrivenBean");

		context.close();

		System.out.println("TopicConnection.createTopicSession(false, javax.jms.Session.DUPS_OK_ACKNOWLEDGE);");
		topicSession = connection.createTopicSession(false, javax.jms.Session.DUPS_OK_ACKNOWLEDGE);

		System.out.println("TopicSession.createPublisher("+topic+");");
		publisher = topicSession.createPublisher(topic);
	}


	public void ejbCreate()
	{
	}

	/**
	 * Describes the instance and its content for debugging purpose
	 *
	 * @return Debugging information about the instance and its content
	 **/
	public String toString()
	{
		return "TestMessageDrivenBean [ " + " ]";
	}

	public void setMessageDrivenContext(MessageDrivenContext aContext)
		throws
		EJBException
	{
		mContext = aContext;
	}

	public void ejbRemove()
	{
		if(connection==null)
			return;
		try{
			connection.close();
		}
		catch(JMSException e){}
		connection = null;
	}
}
